/* -*- Mode: C; indent-tabs-mode:nil; c-basic-offset: 8-*- */

/*
 *This file is part of MlView.
 *
 *MlView is free software; you can redistribute 
 *it and/or modify it under the terms of 
 *the GNU General Public License as published by the 
 *Free Software Foundation; either version 2, 
 *or (at your option) any later version.
 *
 *GNU MlView is distributed in the hope that it will 
 *be useful, but WITHOUT ANY WARRANTY; 
 *without even the implied warranty of 
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *See the GNU General Public License for more details.
 *
 *You should have received a copy of the 
 *GNU General Public License along with MlView; 
 *see the file COPYING. 
 *If not, write to the Free Software Foundation, 
 *Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *See COPYRIGHT file for copyright information.
 */

#include "mlview-icompletion-widget.h"
#include "mlview-marshal.h"

/**
 *@file
 *The definition of the #MlViewICompletionWidget interface.
 */

enum MlViewICompletionWidgetSignals {
        SELECTED_A_POSSIBLE_CHILD,
        SELECTED_A_POSSIBLE_NEXT_SIBLING,
        SELECTED_A_POSSIBLE_PREV_SIBLING,
        SELECTED_A_POSSIBLE_ATTRIBUTE,
        NB_SIGNALS
};

static guint gv_signals[NB_SIGNALS] = {0};

static void
mlview_icompletion_widget_base_init (gpointer a_klass)
{
        static gboolean initialized = FALSE;
        
        if (initialized)
                return;

        gv_signals [SELECTED_A_POSSIBLE_CHILD] = 
                g_signal_new ("selected-a-possible-child", 
                              MLVIEW_TYPE_ICOMPLETION_WIDGET,
                              G_SIGNAL_RUN_FIRST,
                              0, NULL, NULL,
                              mlview_marshal_VOID__POINTER,
                              G_TYPE_NONE, 1, G_TYPE_POINTER);
        gv_signals [SELECTED_A_POSSIBLE_NEXT_SIBLING] = 
                g_signal_new ("selected-a-possible-next-sibling", 
                              MLVIEW_TYPE_ICOMPLETION_WIDGET,
                              G_SIGNAL_RUN_FIRST,
                              0, NULL, NULL,
                              mlview_marshal_VOID__POINTER,
                              G_TYPE_NONE, 1, G_TYPE_POINTER);
        gv_signals [SELECTED_A_POSSIBLE_PREV_SIBLING] = 
                g_signal_new ("selected-a-possible-prev-sibling", 
                              MLVIEW_TYPE_ICOMPLETION_WIDGET,
                              G_SIGNAL_RUN_FIRST,
                              0, NULL, NULL,
                              mlview_marshal_VOID__POINTER,
                              G_TYPE_NONE, 1, G_TYPE_POINTER);
        gv_signals [SELECTED_A_POSSIBLE_ATTRIBUTE] = 
                g_signal_new ("selected-a-possible-attribute", 
                              MLVIEW_TYPE_ICOMPLETION_WIDGET,
                              G_SIGNAL_RUN_FIRST,
                              0, NULL, NULL,
                              mlview_marshal_VOID__POINTER,
                              G_TYPE_NONE, 1, G_TYPE_POINTER);

        initialized = TRUE;
}

/**
 *Getter/builder of the type identifier of the
 *#MlViewICompletionWidget interface.
 *@return the new type identifier.
 */
GType
mlview_icompletion_widget_get_type (void)
{
        static GType type = 0;
        
        if (!type) {
                static const GTypeInfo info = {
                        sizeof (MlViewICompletionWidget), /*class size*/
                        mlview_icompletion_widget_base_init, /*base_init*/
                        NULL, /*base_finalize*/
                        NULL, /*class init*/
                        NULL, /*class finalize*/
                        NULL, /*class data*/
                        0, /*instance size*/
                        0, /*n_prealloc*/
                        NULL,/*instance_init*/
                } ;
                type = g_type_register_static (G_TYPE_INTERFACE,
                                               "MlViewICompletionWidget",
                                               &info, 0);
                g_type_interface_add_prerequisite (type, GTK_TYPE_WIDGET);
        }
        
        return type;
}

/**
 *Updates the possible attributes list
 *in the completion widget.
 *@param a_widget the current instance of #MlViewICompletionWidget.
 *@param a_list the list of possible attributes (gchar *).
 */
void 
mlview_icompletion_widget_update_feasible_attributes_list (MlViewICompletionWidget *a_widget,
							   GList *a_list)
{
        g_return_if_fail (a_widget && MLVIEW_ICOMPLETION_WIDGET_GET_IFACE (a_widget));

        MLVIEW_ICOMPLETION_WIDGET_GET_IFACE (a_widget)->update_feasible_attributes_list
          (a_widget, a_list);
}

/**
 *Updates the possible next siblings list
 *in the completion widget.
 *@param a_widget the current instance of #MlViewICompletionWidget.
 *@param a_list the list of possible next siblings (gchar *).
 */
void
mlview_icompletion_widget_update_feasible_next_siblings_list (MlViewICompletionWidget *a_widget,
							      GList *a_list)
{
        g_return_if_fail (a_widget && MLVIEW_ICOMPLETION_WIDGET_GET_IFACE (a_widget));
        
        MLVIEW_ICOMPLETION_WIDGET_GET_IFACE (a_widget)->update_feasible_next_siblings_list 
                (a_widget, a_list);
}

/**
 *Updates the possible previous siblings list
 *in the completion widget.
 *@param a_widget the current instance of #MlViewICompletionWidget.
 *@param a_list the list of possible previous siblings (gchar *).
 */
void 
mlview_icompletion_widget_update_feasible_prev_siblings_list (MlViewICompletionWidget *a_widget,
							      GList *a_list)
{
        g_return_if_fail (a_widget && MLVIEW_ICOMPLETION_WIDGET_GET_IFACE (a_widget));
        
        MLVIEW_ICOMPLETION_WIDGET_GET_IFACE (a_widget)->update_feasible_prev_siblings_list
                (a_widget, a_list);
}

/**
 *Updates the possible children list
 *in the completion widget.
 *@param a_widget the current instance of #MlViewICompletionWidget.
 *@param a_list the list of possible previous siblings (gchar *).
 */
void
mlview_icompletion_widget_update_feasible_children_list (MlViewICompletionWidget *a_widget,
							 GList *a_list)
{
        g_return_if_fail (a_widget && MLVIEW_ICOMPLETION_WIDGET_GET_IFACE (a_widget));

        MLVIEW_ICOMPLETION_WIDGET_GET_IFACE (a_widget)->update_feasible_children_list
                (a_widget, a_list);
}

/**
 *Empty all the completion lists
 *in the completion widget.
 *@param a_widget the current instance of #MlViewICompletionWidget.
 */
void 
mlview_icompletion_widget_clear_lists (MlViewICompletionWidget *a_widget)
{
        g_return_if_fail (a_widget && MLVIEW_ICOMPLETION_WIDGET_GET_IFACE (a_widget));

        mlview_icompletion_widget_update_feasible_attributes_list (a_widget, NULL);
        mlview_icompletion_widget_update_feasible_next_siblings_list (a_widget, NULL);
        mlview_icompletion_widget_update_feasible_prev_siblings_list (a_widget, NULL);
        mlview_icompletion_widget_update_feasible_children_list (a_widget, NULL);
}
